#!/usr/bin/env python2
# -*- coding: utf-8 -*-
from duinobot import *
# Importamos pygame.joystick para poder controlar el servo con los
# botones de acción.
import pygame.joystick

# Inicializar el módulo de joysticks si es necesario.
if not pygame.joystick.get_init():
    pygame.joystick.init()
joystick = pygame.joystick.Joystick(0)


board = Board('/dev/ttyUSB0')
robot = Robot(board, 1)

# En este caso el servomotor está conectado al pin digital 9 del
# robot.
robot.configServo(9)

# Botones: estos números corresponden a los botones a la derecha
# del modelo de joystick que pudimos probar.
abrir_brazo = 1         # 1 es el botón de acción izquierdo
cerrar_brazo = 3        # 3 es el derecho
descanzar_brazo = 2     # 2 es el de abajo
salir = 9               # 9 es el botón salir

# Ejes: pygame identifica los ejes del d-pad con estos números.
vertical = 1
horizontal = 0

# Estado: Usamos estas variables para controlar cuando el robot
# debe detenerse, la condición para detener el robot depende
# de qué tipo de movimiento está realizando en este momento.
moviendose_eje_vertical = False
moviendose_eje_horizontal = False

print('A continuación puede utilizar el Joystick')
print('Use las teclas de dirección para mover el robot')
print('El botón de acción izquierdo abre el brazo y el derecho lo cierra')
print('Use el botón de abajo para poner el brazo en una posición de descanzo '
      'mientras no lo utiliza para evitar forzar el motor')
print('Si presiona el botón start el programa termina')

# Inicializamos el joystick a usar para que pygame reporte los eventos
# generados por el mismo.
joystick.init()
seguir = True
while seguir:
    for event in pygame.event.get():
        if event.type == pygame.JOYBUTTONDOWN:
            if event.button == salir:
                seguir = False
            elif event.button == abrir_brazo:
                robot.moveServo(9, 0)
                wait(0.2)  # esperamos un tiempo para que el servo se mueva
            elif event.button == cerrar_brazo:
                robot.moveServo(9, 180)
                wait(0.2)
            elif event.button == descanzar_brazo:
                robot.moveServo(9, 90)
                wait(0.2)
        elif event.type == pygame.JOYAXISMOTION:
            if event.axis == vertical:
                if event.value < -0.5:
                    # Arriba
                    robot.forward(60)
                    moviendose_eje_vertical = True
                elif event.value > 0.5:
                    # Abajo
                    robot.backward(60)
                    moviendose_eje_vertical = True
            elif event.axis == horizontal:
                if event.value < -0.5:
                    # Izquierda
                    robot.turnLeft(60)
                    moviendose_eje_horizontal = True
                elif event.value > 0.5:
                    # Derecha
                    robot.turnRight(60)
                    moviendose_eje_horizontal = True
            if abs(event.value) <= 0.5:
                # Para detener el robot es necesario conocer el estado
                # actual.
                if event.axis == vertical and moviendose_eje_vertical:
                    robot.stop()
                    moviendose_eje_vertical = False
                if event.axis == horizontal and moviendose_eje_horizontal:
                    robot.stop()
                    moviendose_eje_horizontal = False

board.exit()
pygame.quit()



from duinobot import *

try:
    #b = TCPBoard('localhost')
    b = TCPBoard()
except NameError:
    b = Board()
r = Robot(b, 12)

while True:
    i, d = r.getLine()
    print((i, d) if i != d else '({}, {}) son iguales'.format(i, d))
    print(b.report())

b.exit()



BOARDS = {
    'arduino' : {
        'digital' : tuple(x for x in range(14)),
        'analog' : tuple(x for x in range(6)),
        'pwm' : (3, 5, 6, 9, 10, 11),
        'use_ports' : True,
        'disabled' : (0, 1) # Rx, Tx, Crystal
    },
    'arduino_mega' : {
        'digital' : tuple(x for x in range(54)),
        'analog' : tuple(x for x in range(16)),
        'pwm' : tuple(x for x in range(2,14)),
        'use_ports' : True,
        'disabled' : (0, 1) # Rx, Tx, Crystal
    },
    'duinobot' : {
        'digital' : tuple(x for x in range(19)),
        'analog' : tuple(x for x in range(6)),
        'pwm' : (5, 6, 9),
        'use_ports' : True,
        'disabled' : (0, 1, 3, 4, 8)
    }    
}


from pyfirmata import *
from boards import BOARDS

__version__ = '0.9.5'  # Don't forget to change in setup.py!

# shortcut classes

class Arduino(Board):
    """
    A board that will set itself up as a normal Arduino.
    """
    def __init__(self, *args, **kwargs):
        args = list(args)
        args.append(BOARDS['arduino'])
        super(Arduino, self).__init__(*args, **kwargs)

    def __str__(self):
        return 'Arduino %s on %s' % (self.name, self.sp.port)


class ArduinoMega(Board):
    """
    A board that will set itself up as an Arduino Mega.
    """
    def __init__(self, *args, **kwargs):
        args = list(args)
        args.append(BOARDS['arduino_mega'])
        super(ArduinoMega, self).__init__(*args, **kwargs)

    def __str__(self):
        return 'Arduino Mega %s on %s' % (self.name, self.sp.port)
        
class DuinoBot(Board):
    """
    A board that wil set itself up as an DuinoBot.
    """
    def __init__(self, *args, **kwargs):
        args = list(args)
        args.append(BOARDS['duinobot'])
        super(DuinoBot, self).__init__(*args, **kwargs)
    
    def __str__(self):
        return 'DuinoBot %s on %s' % (self.name, self.sp.port)


class TCPDuinoBot(TCPBoard):
    """
    A board that wil set itself up as an DuinoBot.
    """
    def __init__(self, *args, **kwargs):
        args = list(args)
        args.append(BOARDS['duinobot'])
        super(TCPDuinoBot, self).__init__(*args, **kwargs)

    def __str__(self):
        return 'TCPDuinoBot %s on %s:%s' % (self.name, self.ip, self.port)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# http://www.jamendo.com/es/track/324954/xxl080808
# http://www.jamendo.com/es/track/247586/razorman-mechanical
from duinobot import *
import os
b = Board()
r1 = Robot(b, 5)
r2 = Robot(b, 18)

pid = os.fork()
if pid == 0:
    salida = os.open('/dev/null', os.O_WRONLY)
    os.dup2(salida, 2)
    os.dup2(salida, 1)
    os.execlp('mplayer',
              'mplayer',
              '-nojoystick',
              'ArenaOfElectronicMusicRazormanMechanical.mp3')


def uno(t):
    lr1, lr2 = r1, r2
    while t > 0:
        lr1.forward(80)
        lr2.turnRight(40, 0.5)
        lr1.backward(80)
        lr2.turnLeft(40, 0.5)
        lr1.stop()
        lr1, lr2 = lr2, lr1
        t -= 1


def dos(t):
    lr1, lr2 = r1, r2

    while t > 0:
        lr1.turnRight(60)
        lr2.turnLeft(70, 0.5)

        lr2.backward(30)
        lr1.forward(70, 0.5)
        lr2.stop()
        lr1, lr2 = lr2, lr1
        t -= 1


def tres(t):
    lr1, lr2 = r1, r2
    while t > 0:
        lr2.turnRight(60)
        lr1.forward(30, 4)

        lr1.backward(30)
        lr2.turnLeft(60, 4)
        lr1.stop()
        t -= 8


def pausa(t):
    print "acomodar Robots!!"
    r1.stop()
    r2.stop()
    lr1, lr2 = r1, r2
    sonido = 0.9
    while t > 0 and sonido > 0:
        lr1.beep(200, sonido)
        lr2.beep(1200, sonido)
        lr1, lr2 = lr2, lr1
        t -= 2*sonido
        sonido -= 0.1
    # if t != 0:
    #     wait(t)


def punchi(t):
    lr1, lr2 = r1, r2
    while t > 0:
        lr1.beep(200)
        lr1.turnLeft(100)
        lr2.turnRight(100, .5)
        lr1.beep(0)
        lr1.stop()
        lr1, lr2 = lr2, lr1
        t -= 0.5

pasos = [
    (27, uno),
    (47 - 28, dos),
    (55 - 47, pausa),
    (65 - 55, tres),
    (75 - 65, dos),
    (81 - 75, punchi),
    (91 - 81, pausa),
    (110 - 91, uno),
    (120 - 110, punchi),
]

for tiempo, paso in pasos:
    print(str(paso))
    paso(tiempo)


b.exit()



#!/bin/python
# -*- coding: utf-8 -*-
###############################################################################
# duinobot-interactive
# Conjunto de clases python para permitir que el robot Multiplo Duinobot N6
# pueda programarse en forma interactiva, sin necesidad de bajar firmware al
# robot. Sin embargo es necesario el soporte del firmware del lado del robot
#
# Copyright (C) 2012 Fernando López <flopez AT linti.unlp.edu.ar>
# Copyright (C) 2011 David Vilaseca <http://www.robotgroup.com.ar>
# Copyright (C) 2011 Joaquín Bogado <jbogado en linti.unlp.edu.ar>
#
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the RobotGroup Multiplo Pacifist License (RMPL)
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the RMPL license along with this program.
# If not, see <multiplo.com.ar/soft/Mbq/Lic.Minibloq.ESP.pdf>.
###############################################################################

from pyfirmata import DuinoBot, TCPDuinoBot, util, SERVO_CONFIG
import time
import re
import os
import threading
from datetime import datetime, timedelta
import itertools

A0, A1, A2, A3, A4, A5 = range(14, 20)
MOVE_SERVO = 0x0A
EXTENDED_PIN_MODE = 0x0B


class Board(object):
    MOTOR_DELAY_TB6612 = timedelta(0, 0, 100000)
    lock = threading.Lock()

    def __ignore_motors(self, robot_id, left=None, right=None):
        '''Para evitar dañar las placas los cambios de velocidad,
        principalmente si invierten el sentido de giro de los motores,
        deben limitarse a 100ms por recomendación del fabricante.'''
        now = datetime.now()
        old_time = self._last_move.get(
            robot_id, now - timedelta(0, 1, 0)
        )

        if left == 0 and right == 0:
            # Siempre dejar pasar los stop() sin registrarlos
            return False
        if now - old_time <= self.MOTOR_DELAY_TB6612:
            return True
        self._last_move[robot_id] = now
        return False

    def __init__(self, device='/dev/ttyUSB0'):
        '''Inicializa el dispositivo de conexion con el/los robot/s'''
        self.board = DuinoBot(device)
        self._generic_initialization()

    def _generic_initialization(self):
        it = util.Iterator(self.board)
        # FIXME: En Python > 2.5 se puede cambiar por it.daemon = True
        # http://stackoverflow.com/questions/17650754
        # /how-can-i-capture-ctrl-d-in-python-interactive-console
        it.setDaemon(True)
        it.start()
        self.board.pass_time(0.1)
        self._last_move = dict()

    def __del__(self):
        self.exit()

    def __getattribute__(self, name):
        '''Permitir multithreading para evitar problemas con el método
        senses'''
        Board.lock.acquire()
        try:
            res = object.__getattribute__(self, name)
        except:
            Board.lock.release()
            raise
        Board.lock.release()
        return res

    def motor0(self, vel, robotid):
        if vel >= 0 and vel <= 100\
                and not self.__ignore_motors(robotid, vel):
            self.board.send_sysex(1, [int(vel), robotid])

    def motor1(self, vel, robotid):
        if vel >= 0 and vel <= 100\
                and not self.__ignore_motors(robotid, right=vel):
            self.board.send_sysex(2, [int(vel), robotid])

    def motors(self, vel1, vel2, seconds=-1, robotid=0):
        if abs(vel1) <= 100 and abs(vel2) <= 100\
                and not self.__ignore_motors(robotid, vel1, vel2):
            self.board.send_sysex(
                4,
                [int(abs(vel1)), int(abs(vel2)), 1
                    if vel1 > 0 else 0, 1 if vel2 > 0 else 0, robotid])
            if seconds != -1:
                self.board.pass_time(seconds)
                self.motors(0, 0, -1, robotid)

    def ping(self, robotid):
        self.board.nearest_obstacle[robotid] = None
        self.board.send_sysex(3, [robotid])
        # A veces hay que esperar más de 0.04 segundos
        for i in xrange(6):
            # wait 20ms (ping delay) + 20ms (comm)
            self.board.pass_time(0.04)
            if self.board.nearest_obstacle[robotid] is not None:
                return self.board.nearest_obstacle[robotid]
        return -1

    def sleep(self, time):
        self.board.pass_time(time)

    def exit(self):
        self.board.exit()

    def analog(self, ch, samples=1, robotid=0):
        self.board.send_sysex(6, [ch, samples, robotid])
        self.board.pass_time(0.04)
        return self.board.analog_value[robotid]

    def battery(self, robotid):
        self.board.send_sysex(6, [6, 1, robotid])
        self.board.pass_time(0.04)
        return self.board.analog_value[robotid]*5.0/1024

    def digital(self, pin, robotid):
        self.board.send_sysex(7, [pin, robotid])
        self.board.pass_time(0.04)
        return self.board.digital_value[robotid]

    def set_pin_mode(self, pin, mode, robotid):
        self.board.send_sysex(EXTENDED_PIN_MODE, [pin, mode, robotid])

    def beep(self, freq=0, microseconds=0, robotid=0):
        hi = freq >> 7
        lo = freq % 128
        if freq != 0:
            if microseconds == 0:
                self.board.send_sysex(5, [hi, lo, robotid])
            else:
                self.board.send_sysex(
                    5, [hi, lo, int(microseconds*1000), robotid])
                # self.board.pass_time(microseconds)
        else:
            self.board.send_sysex(5, [robotid])

    def report(self):
        self.board.live_robots = [0 for i in range(128)]
        self.board.send_sysex(9, [])
        self.board.pass_time(0.5)
        robotlist = []
        for i in range(128):
            if self.board.live_robots[i]:
                robotlist.append(i)
        return robotlist

    def set_id(self, new_id, robotid):
        self.board.send_sysex(8, [new_id, robotid])
        self.board.pass_time(0.02)

    def getLine(self, robotid):
        a = self.analog(18, robotid=robotid)
        b = self.analog(19, robotid=robotid)
        return (a, b)

    def getWheels(self, robotid):
        a = self.analog(16, robotid=robotid)
        b = self.analog(17, robotid=robotid)
        return (a, b)

    wait = sleep


class TCPBoard(Board):
    def __init__(self, robot_ip='192.168.4.1', port=1234):
        self.board = TCPDuinoBot(robot_ip, port)
        self._generic_initialization()


class Robot(object):
    def __init__(self, board, robotid=0):
        '''Inicializa el robot y lo asocia con la placa board.'''
        self.robotid = robotid
        self.board = board
        self.name = ''
        self.pins = dict()

    # MOVIMIENTO
    def forward(self, vel=50, seconds=-1):
        '''El robot avanza con velocidad vel durante seconds segundos.'''
        self.board.motors(vel, vel, seconds, self.robotid)

    def backward(self, vel=50, seconds=-1):
        '''El robot retrocede con velocidad vel durante seconds segundos.'''
        self.forward(-vel, seconds)

    def motors(self, vel1, vel2, seconds=-1):
        '''Permite mover las ruedas independientemente,
        con velocidad vel1 para un motor y vel2 para el otro motor,
        durante second segundos.
        '''
        self.board.motors(vel1, vel2, seconds, self.robotid)

    def turnLeft(self, vel=50, seconds=-1):
        '''El robot gira a la izquierda con velocidad vel durante seconds
        segundos.
        '''
        self.board.motors(vel, -vel, seconds, self.robotid)

    def turnRight(self, vel=50, seconds=-1):
        '''El robot gira a la derecha con velocidad vel durante seconds
        segundos.
        '''
        self.board.motors(-vel, vel, seconds, self.robotid)

    def stop(self):
        '''Detiene todo movimiento del robot inmediatamente.'''
        self.board.motors(0, 0, robotid=self.robotid)

    # SENSORES
    def getLine(self):
        '''Devuelve los valores de los sensores de linea.'''
        return self.board.getLine(self.robotid)

    def getWheels(self):
        '''Devuelve el valor de los sensores de las ruedas.'''
        return self.board.getWheels(self.robotid)

    def getWheelsB(self):
        (r, l) = self.getWheels()
        return (r/500, l/500)

    def getObstacle(self, distance=10):
        '''Devuelve True si hay un obstaculo a menos de distance
         centimetros del robot.'''
        ping = self.ping()
        if ping < 0:
            # Consideramos que una lectura fallida
            # del sensor retorna -1
            return False
        return ping < distance

    def battery(self):
        '''Devuelve el voltaje de las baterías del robot.'''
        return self.board.battery(self.robotid)

    def ping(self):
        '''Devuelve la distancia en centimetros al objeto frente al robot.'''
        return self.board.ping(self.robotid)

    def beep(self, freq=200, seconds=0):
        '''Hace que el robot emita un pitido con frecuencia freq durante
        seconds segundos.
        '''
        self.board.beep(freq, robotid=self.robotid)
        if seconds > 0:
            self.board.wait(seconds)
            self.board.beep(0, robotid=self.robotid)

    def sensors(self):
        '''Imprime informacion de los sensores.'''
        print 'Línea = ' + str(self.getLine())
        print 'Ruedas = ' + str(self.getWheels())
        print 'Obstaculo mas cercano = ' + str(self.ping()) + ' cm.'
        print 'Bateria = ' + str(self.battery()) + ' v.'

    # Identificadores
    def setId(self, newid):
        '''Setea el robotid'''
        self.board.set_id(newid, self.robotid)
        self.robotid = newid

    def setName(self, name):
        '''Setea el nombre para el robot.'''
        self.name = str(name)

    def getId(self):
        '''Devuelve el robotid.'''
        return self.robotid

    def getName(self):
        '''Devuelve el nombre del robot.'''
        return self.name

    # Otras funciones
    def speak(self, msj):
        '''Imprime en la terminal el mensaje msj.'''
        print msj

    def configServo(self, pin, min_pulse=544, max_pulse=2400):
        data = itertools.chain([pin],
                               util.to_two_bytes(min_pulse),
                               util.to_two_bytes(max_pulse),
                               [self.robotid])
        self.board.board.send_sysex(SERVO_CONFIG, data)

    def moveServo(self, pin, angle):
        '''Pasa un ángulo entre 0 y 180 grados al servo conectado
        al pin indicado'''
        data = itertools.chain([pin], reversed(util.to_two_bytes(angle)),
                               [self.robotid])
        self.board.board.send_sysex(MOVE_SERVO, data)

from senses import *
Robot.senses = senses


def wait(seconds):
    '''Produce un retardo de seconds segundos en los que el robot no
    hace nada.
    '''
    time.sleep(seconds)

__devPattern = re.compile(r"^ttyUSB\d+$")


def boards():
    matching = filter(__devPattern.match, os.listdir("/dev"))
    return map(lambda s: "/dev/" + s, matching)



# -*- encoding: utf-8 -*-

###############################################################################
# joystick.py
#
# Copyright (C) 2012 Fernando López <soportelihuen AT linti.unlp.edu.ar>
#
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the RobotGroup Multiplo Pacifist License (RMPL)
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the RMPL license along with this program.
# If not, see <multiplo.com.ar/soft/Mbq/Lic.Minibloq.ESP.pdf>.
###############################################################################

try:
    import pygame
    # Inicializar solamente el Joystick es más rápido pero suele fallar
    pygame.init()
except ImportError:
    print('Para usar joysticks tenés que instalar pygame')


def joysticks():
    "Retorna una lista joysticks conectados"
    Joystick._Joystick__refresh()
    joys = []
    for id in range(pygame.joystick.get_count()):
        j = pygame.joystick.Joystick(id)
        joys.append((id, j.get_name(),))
        j.quit()
    return joys


class Joystick:
    "Permite controlar un robot con un joystick"
    if 'pygame' in globals():
        joystickDetected = pygame.joystick.get_count() > 0
    else:
        joystickDetected = False

    @classmethod
    def __refresh(cls):
        """Esta función permite conectar el joystick aun después de haber
        importado el módulo, pero solo tendrá efecto una vez luego de
        detectar algún joystick (es decir si se conectan o desconectan
        joysticks más tarde la función no tendrá efecto)"""
        if pygame.joystick.get_count() == 0 and not Joystick.joystickDetected:
            pygame.joystick.quit()
            pygame.joystick.init()
            Joystick.joystickDetected = pygame.joystick.get_count() > 0

    def __init__(self, robot, joystickNumber=0):
        """Crea un joystick asociado con un objeto Robot, si
        no se idica el numero de joystick se asume 0"""
        Joystick.__refresh()
        self.js = pygame.joystick.Joystick(joystickNumber)
        self.js.init()
        self.robot = robot

    def play(self):
        """Permite controlar un robot con un joystick analógico o
        digital, cuando se usan los botones el robot
        se mueve a velocidad máxima, si se usa un control analógico
        se puede graduar la velocidad"""
        previous = None  # Permite evitar procesar eventos repetidos
        # Las siguientes 3 variables sirven para evitar conflictos
        # entre los eventos que se generan para cada eje.
        forward = False
        rotate = False
        digitalRotate = False
        while True:
            event = pygame.event.wait()
            if (
                    previous and
                    event.type == pygame.JOYAXISMOTION and
                    event.joy == previous.joy and
                    event.value == previous.value and
                    event.axis == previous.axis):
                pass
            else:
                if event.type == pygame.JOYAXISMOTION:
                    previous = event
                elif event.type == pygame.JOYBUTTONDOWN and event.button == 9:
                    return
                else:
                    continue

                if event.axis == 2 or event.axis == 1:
                    vel = event.value * 100
                    if vel > 100:
                        vel = 100
                    elif vel < -100:
                        vel = -100
                    if event.axis == 1 and not rotate:
                        self.robot.forward(-vel)
                        forward = vel != 0
                    elif event.axis == 2 and not forward and not digitalRotate:
                        self.robot.turnLeft(-vel)
                        rotate = vel != 0
                elif event.axis == 0:
                    digitalRotate = True
                    if event.value > 0:
                        self.robot.turnRight(100)
                    elif event.value < 0:
                        self.robot.turnLeft(100)
                    else:
                        digitalRotate = False
                        self.robot.stop()



import socket
import sys

server = socket.create_connection(('192.168.4.1', 1234))
msj = [0xf0, 0x09, 0xf7]
server.sendall(''.join(map(chr, msj)))

while True:
    byte = ord(server.recv(1))
    sys.stdout.write('0x{:02x} '.format(byte))
    if byte == 0xf7:
        print('')



import serial
import sys
import time

print(sys.argv)
sp = serial.Serial(port=sys.argv[1], baudrate=57600)

while True:
    byte = ord(sp.read())
    if not byte:
        time.sleep(0.2)
        continue
    sys.stdout.write('0x{:02x} '.format(byte))
    print('')



#!/bin/python
# -*- coding: utf-8 -*-

from duinobot import *
from time import *


b = Board('/dev/ttyUSB0')
r = Robot(b, 1)

ctime = time()
while time() - ctime < 30:
    l1, l2 = r.getLine()
    if l1 < 500 or l2 < 500:
        r.backward(30, 0.7)
        r.turnRight(30, 0.7)
    else:
        r.forward(30)
r.stop()
b.exit()



import threading
import serial
import time
import os
import pyfirmata
from boards import BOARDS

def get_the_board(layout=BOARDS['arduino'], base_dir='/dev/', identifier='tty.usbserial',):
    """
    Helper function to get the one and only board connected to the computer
    running this. It assumes a normal arduino layout, but this can be
    overriden by passing a different layout dict as the ``layout`` parameter.
    ``base_dir`` and ``identifier`` are overridable as well. It will raise an
    IOError if it can't find a board, on a serial, or if it finds more than
    one.
    """
    boards = []
    for device in os.listdir(base_dir):
        if device.startswith(identifier):
            try:
                board = pyfirmata.Board(os.path.join(base_dir, device), layout)
            except serial.SerialException:
                pass
            else:
                boards.append(board)
    if len(boards) == 0:
        raise IOError, "No boards found in %s with identifier %s" % (base_dir, identifier)
    elif len(boards) > 1:
        raise IOError, "More than one board found!"
    return boards[0]

class Iterator(threading.Thread):
    def __init__(self, board):
        super(Iterator, self).__init__()
        self.board = board

    def run(self):
        while 1:
            try:
                while self.board.bytes_available() and self.board.running:
                    self.board.iterate()
                time.sleep(0.001)
                if not self.board.running:    
                    return 
            except (AttributeError, serial.SerialException, OSError), e:
                # this way we can kill the thread by setting the board object
                # to None, or when the serial port is closed by board.exit()
                break
            except Exception, e:
                # catch 'error: Bad file descriptor'
                # iterate may be called while the serial port is being closed,
                # causing an "error: (9, 'Bad file descriptor')"
                if getattr(e, 'errno', None) == 9:
                    break
                try:
                    if e[0] == 9:
                        break
                except (TypeError, IndexError):
                    pass
                raise

def to_two_bytes(integer):
    """
    Breaks an integer into two 7 bit bytes.

    >>> for i in range(32768):
    ...     val = to_two_bytes(i)
    ...     assert len(val) == 2
    ...
    >>> to_two_bytes(32767)
    ('\\x7f', '\\xff')
    >>> to_two_bytes(32768)
    Traceback (most recent call last):
        ...
    ValueError: Can't handle values bigger than 32767 (max for 2 bits)

    """
    if integer > 32767:
        raise ValueError, "Can't handle values bigger than 32767 (max for 2 bits)"
    return chr(integer % 128), chr(integer >> 7)

def from_two_bytes(bytes):
    """
    Return an integer from two 7 bit bytes.

    >>> for i in range(32766, 32768):
    ...     val = to_two_bytes(i)
    ...     ret = from_two_bytes(val)
    ...     assert ret == i
    ...
    >>> from_two_bytes(('\\xff', '\\xff'))
    32767
    >>> from_two_bytes(('\\x7f', '\\xff'))
    32767
    """
    lsb, msb = bytes
    try:
        # Usually bytes have been converted to integers with ord already
        return msb << 7 | lsb
    except TypeError:
        # But add this for easy testing
        # One of them can be a string, or both
        try:
            lsb = ord(lsb)
        except TypeError:
            pass
        try:
            msb = ord(msb)
        except TypeError:
            pass
        return msb << 7 | lsb

def two_byte_iter_to_str(bytes):
    """
    Return a string made from a list of two byte chars.

    >>> string, s = 'StandardFirmata', []
    >>> for i in string:
    ...   s.append(i)
    ...   s.append('\\x00')
    >>> two_byte_iter_to_str(s)
    'StandardFirmata'

    >>> string, s = 'StandardFirmata', []
    >>> for i in string:
    ...   s.append(ord(i))
    ...   s.append(ord('\\x00'))
    >>> two_byte_iter_to_str(s)
    'StandardFirmata'
    """
    bytes = list(bytes)
    chars = []
    while bytes:
        lsb = bytes.pop(0)
        try:
            msb = bytes.pop(0)
        except IndexError:
            msb = 0x00
        chars.append(chr(from_two_bytes((lsb, msb))))
    return ''.join(chars)

def str_to_two_byte_iter(string):
    """
    Return a iter consisting of two byte chars from a string.

    >>> string, iter = 'StandardFirmata', []
    >>> for i in string:
    ...   iter.append(i)
    ...   iter.append('\\x00')
    >>> assert iter == str_to_two_byte_iter(string)
     """
    bytes = []
    for char in string:
        bytes += list(to_two_bytes(ord(char)))
    return bytes

def break_to_bytes(value):
    """
    Breaks a value into values of less than 255 that form value when multiplied.
    (Or almost do so with primes)
    Returns a tuple

    >>> break_to_bytes(200)
    (200,)
    >>> break_to_bytes(800)
    (200, 4)
    >>> break_to_bytes(802)
    (2, 2, 200)
    """
    if value < 256:
        return (value,)
    c = 256
    least = (0, 255)
    for i in range(254):
        c -= 1
        rest = value % c
        if rest == 0 and value / c < 256:
            return (c, value / c)
        elif rest == 0 and value / c > 255:
            parts = list(break_to_bytes(value / c))
            parts.insert(0, c)
            return tuple(parts)
        else:
            if rest < least[1]:
                least = (c, rest)
    return (c, value / c)

if __name__ == '__main__':
    import doctest
    doctest.testmod()



from collections import deque
import pyfirmata

class MockupSerial(deque):
    """ 
    A Mockup object for python's Serial. Functions as a fifo-stack. Push to
    it with ``write``, read from it with ``read``.
    
    >>> s = MockupSerial('someport', 4800)
    >>> s.read()
    ''
    >>> s.write(chr(100))
    >>> s.write('blaat')
    >>> s.write(100000)
    >>> s.read(2)
    ['d', 'blaat']
    >>> s.read()
    100000
    >>> s.read()
    ''
    >>> s.read(2)
    ['', '']
    >>> s.close()
    """
    def __init__(self, port, baudrate, timeout=0.02):
        self.port = port or 'somewhere'
        
    def read(self, count=1):
        if count > 1:
            val = []
            for i in range(count):
                try:
                    val.append(self.popleft())
                except IndexError:
                    val.append('')
        else:
            try:
                val = self.popleft()
            except IndexError:
                val = ''
        return val
            
    def write(self, value):
        """
        Appends items flat to the deque. So iterables will be unpacked.
        """
        if hasattr(value, '__iter__'):
            self.extend(value)
        else:
            self.append(value)
            
    def close(self):
        self.clear()
        
    def inWaiting(self):
        return len(self)
        
class MockupBoard(pyfirmata.Board):

    def __init__(self, port, layout, values_dict={}):
        self.sp = MockupSerial(port, 57600)
        self.setup_layout(layout)
        self.values_dict = values_dict
        self.id = 1
        
    def reset_taken(self):
        for key in self.taken['analog']:
            self.taken['analog'][key] = False
        for key in self.taken['digital']:
            self.taken['digital'][key] = False
        
    def update_values_dict(self):
        for port in self.digital_ports:
            port.values_dict = self.values_dict
            port.update_values_dict()
        for pin in self.analog:
            pin.values_dict = self.values_dict
        
class MockupPort(pyfirmata.Port):
    def __init__(self, board, port_number):
        self.board = board
        self.port_number = port_number
        self.reporting = False
        
        self.pins = []
        for i in range(8):
            pin_nr = i + self.port_number * 8
            self.pins.append(MockupPin(self.board, pin_nr, type=pyfirmata.DIGITAL, port=self))

    def update_values_dict(self):
        for pin in self.pins:
            pin.values_dict = self.values_dict
        
class MockupPin(pyfirmata.Pin):
    def __init__(self, *args, **kwargs):
        self.values_dict = kwargs.get('values_dict', {})
        try:
            del kwargs['values_dict']
        except KeyError:
            pass
        super(MockupPin, self).__init__(*args, **kwargs)
    
    def read(self):
        if self.value is None:
            try:
                type = self.port and 'd' or 'a'
                return self.values_dict[type][self.pin_number]
            except KeyError:
                return None
        else:
            return self.value
            
    def get_in_output(self):
        if not self.port and not self.mode: # analog input
            return 'i'
        else:
            return 'o'
            
    def set_active(self, active):
        self.is_active = active
        
    def get_active(self):
        return self.is_active
        
    def write(self, value):
        if self.mode == pyfirmata.UNAVAILABLE:
            raise IOError, "Cannot read from pin %d" \
                           % (self.pin_number)
        if self.mode == pyfirmata.INPUT:
            raise IOError, "%s pin %d is not an output" \
                            % (self.port and "Digital" or "Analog", self.get_pin_number())
        if not self.port:
            raise AttributeError, "AnalogPin instance has no attribute 'write'"
        # if value != self.read():
        self.value = value
        
class Iterator(object):
    def __init__(self, *args, **kwargs):
        pass
    def start(self):
        pass
    def stop(self):
        pass

if __name__ == '__main__':
    import doctest
    doctest.testmod()


# -*- coding: utf-8 -*-
import socket
import sys

def handle(msj, client):
    c = handle.c
    if msj[0] == 0x06:
        response = [0xf0, 0x06]
        response.append(c / 128)
        response.append(c % 128)
        # Robot id
        response.append(12)
        response.append(0xf7)
        response = ''.join(map(chr, response))
        print("Respuesta {}".format(
            ['0x{:02x}'.format(ord(b)) for b in response]
        ))
        client.sendall(response)
    c = (c + 1) % 2**14
    handle.c = c
handle.c = 0

server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
server.bind(('127.0.0.1', 1234))
server.listen(1)

START = 0xf0
END = 0xf7
DELIM = (START, END)

while True:
    client, clientaddr = server.accept()
    try:
        while True:
            sys.stderr.write('Esperando 0xf0\n')
            data = ord(client.recv(1))
            while data != START:
                sys.stderr.write("0x{:02x} ".format(data))
                data = ord(client.recv(1))
            sys.stderr.write('\n')
            assert data != END
            if data == START:
                sys.stderr.write('Leyendo y esperando 0xf7\n')
                msj = []
                data = ord(client.recv(1))
                while data not in DELIM:
                    sys.stderr.write("0x{:02x} ".format(data))
                    msj.append(data)
                    data = ord(client.recv(1))
                sys.stderr.write('\n')
                if data == END:
                    print(msj)
                    handle(msj, client)
                else:
                    print('Mensaje incompleto {}'.format(msj))
    except TypeError as e:
        print(e.message)
        client.close()
        continue
    except:
        client.close()
        server.close()
        raise




#!/bin/python
# -*- coding: utf-8 -*-
from duinobot import *
from time import *


b = Board('/dev/ttyUSB0')
r = Robot(b, 1)

ctime = time()
# Sigue líneas negras sobre fondos blancos
r.forward(30)
r.senses()
while time() - ctime < 30:
    izq, der = r.getLine()
    if izq > 500:
        r.turnRight(30)
    elif der > 500:
        r.turnLeft(30)
    else:
        r.forward(30)
    wait(0.3)  # Espera al menos 0.3 segundos antes de volver a sensar
r.stop()
b.exit()



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
from duinobot import *

board = Board('/dev/ttyUSB0')
robot = Robot(board, 1)

if not joysticks():
    print('No se encontró ningún joystick')
    board.exit()
    exit()

print('Ahora puede controlar el robot usando el joystick, '
      'presione el botón "start" para terminar el programa.')
# Si esto falla ejecutar joysticks() de forma interactiva para ver
# cuántos joysticks se detectaron
# si hay más de uno, indique el número del que desea utilizar como
# segundo argumento de Joystick().
joystick = Joystick(robot)
joystick.play()

board.exit()



# -*- encoding: utf-8 -*-
# Solución del ejercicio 4 de la sección 3.2.3.
from duinobot import *

b = Board("/dev/ttyUSB0")

robot1 = Robot(b, 1)
robot2 = Robot(b, 10)


def carrera(r1, r2, tiempo):
    r1.forward(100)
    r2.forward(100)
    wait(tiempo)
    r1.stop()
    r2.stop()

carrera(robot1, robot2, 3)

b.exit()



#!/usr/bin/python
# -*- encoding: utf-8 -*-
###############################################################################
# Copyright (C) 2012 Fernando López <flopez AT linti.unlp.edu.ar>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the RobotGroup Multiplo Pacifist License (RMPL)
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the RMPL license along with this program.
# If not, see <multiplo.com.ar/soft/Mbq/Lic.Minibloq.ESP.pdf>.
###############################################################################


class SensesModel():
    def __init__(self):
        self.wheelLeft = tk.StringVar()
        self.wheelRight = tk.StringVar()
        self.ping = tk.StringVar()
        self.battery = tk.StringVar()


class SensesGUI():
    def __init__(self, root, model):
        self.root = root
        root.title("Senses")
        self.frame = tk.Frame(root)
        self.frame.pack(expand=1, padx=5, pady=5)
        # self.frame["padding"] = 5
        label = tk.Label(self.frame)
        label["text"] = "Ping:"
        label.grid(row=0, sticky=tk.W)
        self.ping = tk.Entry(self.frame)
        self.ping.grid(row=0, column=1)
        label = tk.Label(self.frame)
        label["text"] = "Sensores de las ruedas:"
        label.grid(row=1, columnspan=2)
        self.wheelLeft = tk.Entry(self.frame)
        self.wheelLeft.grid(row=2, column=0)
        self.wheelRight = tk.Entry(self.frame)
        self.wheelRight.grid(row=2, column=1)
        label = tk.Label(self.frame)
        label["text"] = "Medidor de batería: "
        label.grid(row=3, sticky=tk.W)
        self.battery = tk.Entry(self.frame)
        self.battery.grid(row=3, column=1)
        self.model = model
        self._setEntries()

    def _setEntries(self):
        self.wheelLeft["textvariable"] = self.model.wheelLeft
        self.wheelRight["textvariable"] = self.model.wheelRight
        self.ping["textvariable"] = self.model.ping
        self.battery["textvariable"] = self.model.battery


def _updateModel(model, messages, root):
    values = messages.get()
    if not values:
        return
    wleft, wright = values["wheels"]
    model.ping.set(str(values["ping"]))
    model.wheelLeft.set(str(wleft))
    model.wheelRight.set(str(wright))
    model.battery.set(str(values["battery"]))
    root.after(1000, _updateModel, model, messages, root)


def _sensesDialog(messages):
    global tk
    import Tkinter  # Tkinter must be imported in the process that uses it
    tk = Tkinter
    root = tk.Tk()
    model = SensesModel()
    gui = SensesGUI(root, model)
    root.after_idle(_updateModel, model, messages, root)
    root.mainloop()
    root.quit()


def _sendSensorsValues(robot):
    messages = multiprocessing.Queue()
    senses = multiprocessing.Process(target=_sensesDialog,
                                     args=(messages,))
    senses.daemon = True
    senses.start()
    while senses.is_alive():
        values = {
            "wheels": robot.getWheels(),
            "ping": robot.ping(),
            "battery": robot.battery()
        }
        # values = {
        #     "line": (1,1),
        #     "ping": int(time.time() % 100),
        #     "battery": int(time.time() % 100)
        # }
        messages.put(values)
        time.sleep(1)
    senses.join()

import platform
mayor, minor, revision = platform.python_version_tuple()
if mayor != '2' or minor < '6':
    print("El módulo senses precisa una versión de Python mayor" +
          "o igual a 2.6 y menor a 3")

    def senses(robot):
        print("Función no disponible en esta versión de Python")
else:
    import threading
    import multiprocessing
    import time

    def senses(robot):
        update = threading.Thread(target=_sendSensorsValues, args=(robot,))
        update.setDaemon(True)
        update.start()

if __name__ == "__main__":
    from duinobot import *
    b = Board("/dev/ttyUSB0")
    r = Robot(b, 1)
    senses(r)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from distutils.core import setup
import os.path
import glob

setup(name='duinobot',
      version='0.14',
      description='API en Python para el robot Multiplo N6',
      author='Fernando Lopez, Sofia Martin, Augusto Guerrero, Joaquin Bogado',
      author_email='soportelihuen@linti.unlp.edu.ar',
      url='http://lihuen.linti.unlp.edu.ar',
      packages=['duinobot', 'duinobot.pyfirmata'],
      package_dir={'duinobot': '.'},
      data_files=[('usr/share/doc/duinobot/ejemplos',
                   glob.glob(os.path.join('ejemplos', '*.py')))],
      requires=['pyserial', 'pygame'],
      classifiers=[
          "Environment :: Console",
          "Intended Audience :: Developers",
          "Intended Audience :: Education",
          "Operating System :: OS Independent",
          "Programming Language :: Python",
          "Topic :: Education",
          "Programming Language :: Python :: 2.6",
          "Programming Language :: Python :: 2.7"
      ])



import duinobot
import pyfirmata
import time

rid = 16

b = duinobot.TCPBoard()
print(b.report)
r = duinobot.Robot(b, 16)
r.forward(100, 1)
r.backward(100, 1)
r.turnLeft(100, 1)
r.turnRight(100, 1)
print(r.ping())
print(r.getLine())
print('Digital {}'.format(b.digital(2, rid)))
b.set_pin_mode(2, pyfirmata.pyfirmata.INPUT, rid)
print('Digital {}'.format(b.digital(2, rid)))

for i in range(20):
    print('Digital {}'.format(b.digital(2, rid)))
    time.sleep(5)



from duinobot import *
import sys

b = Board()
# id del robot
idr = 1
r = Robot(b, idr)

while 1:
    for i in range(6):
        print b.analog(i, robotid=idr),
        print '\t',
    print '\n',
    if r.ping() < 50:
        break


b.exit()



# -*- coding: utf-8 -*-
import serial
import inspect
import time
import itertools
import os
from util import two_byte_iter_to_str, to_two_bytes
import socket
import select


# Message command bytes - straight from Firmata.h
DIGITAL_MESSAGE = 0x90      # send data for a digital pin
ANALOG_MESSAGE = 0xE0       # send data for an analog pin (or PWM)
DIGITAL_PULSE = 0x91        # SysEx command to send a digital pulse

# PULSE_MESSAGE = 0xA0      # proposed pulseIn/Out msg (SysEx)
# SHIFTOUT_MESSAGE = 0xB0   # proposed shiftOut msg (SysEx)
REPORT_ANALOG = 0xC0        # enable analog input by pin #
REPORT_DIGITAL = 0xD0       # enable digital input by port pair
START_SYSEX = 0xF0          # start a MIDI SysEx msg
SET_PIN_MODE = 0xF4         # set a pin to INPUT/OUTPUT/PWM/etc
END_SYSEX = 0xF7            # end a MIDI SysEx msg
REPORT_VERSION = 0xF9       # report firmware version
SYSTEM_RESET = 0xFF         # reset from MIDI
QUERY_FIRMWARE = 0x79       # query the firmware name

# extended command set using sysex (0-127/0x00-0x7F)
# 0x00-0x0F reserved for user-defined commands */
SYSEX_PING = 0x03
ANALOG_INPUT_REQUEST = 0x06
DIGITAL_INPUT_REQUEST = 0x07
BROADCAST_REPORT = 0x09


SERVO_CONFIG = 0x70         # set max angle, minPulse, maxPulse, freq
STRING_DATA = 0x71          # a string message with 14-bits per char
SHIFT_DATA = 0x75           # a bitstream to/from a shift register
I2C_REQUEST = 0x76          # send an I2C read/write request
I2C_REPLY = 0x77            # a reply to an I2C read request
I2C_CONFIG = 0x78           # config I2C settings such as delay times and power pins
REPORT_FIRMWARE = 0x79      # report name and version of the firmware
SAMPLING_INTERVAL = 0x7A    # set the poll rate of the main loop
SYSEX_NON_REALTIME = 0x7E   # MIDI Reserved for non-realtime messages
SYSEX_REALTIME = 0x7F       # MIDI Reserved for realtime messages


# Pin modes.
# except from UNAVAILABLE taken from Firmata.h
UNAVAILABLE = -1
INPUT = 0          # as defined in wiring.h
OUTPUT = 1         # as defined in wiring.h
ANALOG = 2         # analog pin in analogInput mode
PWM = 3            # digital pin in PWM output mode
SERVO = 4          # digital pin in SERVO mode

# Pin types
DIGITAL = OUTPUT   # same as OUTPUT below
# ANALOG is already defined above

# Time to wait after initializing serial, used in Board.__init__
BOARD_SETUP_WAIT_TIME = 5


class PinAlreadyTakenError(Exception):
    pass


class InvalidPinDefError(Exception):
    pass


class NoInputWarning(RuntimeWarning):
    pass

class Board(object):
    """The Base class for any board."""
    firmata_version = None
    firmware = None
    firmware_version = None
    _command_handlers = {}
    _command = None
    _stored_data = []
    _parsing_sysex = False
    nearest_obstacle = [-1 for i in range(128)]
    analog_value = [-1 for i in range(128)]
    digital_value = [-1 for i in range(128)]
    live_robots = [0 for i in range(128)]

    def __init__(self, port, layout, baudrate=57600, name=None):
        try:
            self.sp = serial.Serial(port, baudrate)
            if name is None:
                name = port
            self._generic_initialization(layout, name)
        except serial.SerialException:
            if os.path.exists(port) and not os.access(port, os.R_OK | os.W_OK):
                print ("No tiene permisos para acceder al dispositivo,"
                       " verifique si su usuario pertenece al grupo dialout")
            else:
                print ("No es posible conectarse al robot, por favor enchufe "
                       "y configure el XBee")
            raise  # re-raise the exception to allow the caller to handle this

    def _generic_initialization(self, layout, name):
        # Allow 5 secs for Arduino's auto-reset to happen
        # Alas, Firmata blinks its version before printing it to serial
        # For 2.3, even 5 seconds might not be enough.
        # TODO Find a more reliable way to wait until the board is ready
        self.pass_time(BOARD_SETUP_WAIT_TIME)
        self.name = name
        self.setup_layout(layout)
        # Iterate over the first messages to get firmware data
        while self.bytes_available():
            self.iterate()
        # TODO Test whether we got a firmware name and version,
        # otherwise there
        # probably isn't any Firmata installed
        self.running = 1

    def __str__(self):
        return "Board %s on %s" % (self.name, self.sp.port)

    def __del__(self):
        """
        The connection with the a board can get messed up when a script is
        closed without calling board.exit() (which closes the serial
        connection). Therefore also do it here and hope it helps.
        """
        self.exit()

    def send_as_two_bytes(self, val):
        self.sp.write(chr(val % 128) + chr(val >> 7))

    def setup_layout(self, board_layout):
        """
        Setup the Pin instances based on the given board layout. Maybe it will
        be possible to do this automatically in the future, by polling the
        board for its type.
        """
        # Create pin instances based on board layout
        self.analog = []
        for i in board_layout['analog']:
            self.analog.append(Pin(self, i))

        self.digital = []
        self.digital_ports = []
        for i in xrange(0, len(board_layout['digital']), 8):
            num_pins = len(board_layout['digital'][i:i + 8])
            port_number = i / 8
            self.digital_ports.append(Port(self, port_number, num_pins))

        # Allow to access the Pin instances directly
        for port in self.digital_ports:
            self.digital += port.pins

        # Setup PWM pins
        for i in board_layout['pwm']:
            self.digital[i].PWM_CAPABLE = True

        # Disable certain ports like Rx/Tx and crystal ports
        for i in board_layout['disabled']:
            self.digital[i].mode = UNAVAILABLE

        # Create a dictionary of 'taken' pins. Used by the get_pin method
        self.taken = { 'analog' : dict(map(lambda p: (p.pin_number, False), self.analog)),
                       'digital' : dict(map(lambda p: (p.pin_number, False), self.digital)) }

        # Setup default handlers for standard incoming commands
        self.add_cmd_handler(ANALOG_MESSAGE, self._handle_analog_message)
        self.add_cmd_handler(DIGITAL_MESSAGE, self._handle_digital_message)
        self.add_cmd_handler(REPORT_VERSION, self._handle_report_version)
        self.add_cmd_handler(REPORT_FIRMWARE, self._handle_report_firmware)
        self.add_cmd_handler(SYSEX_PING, self._handle_sysex_ping)
        self.add_cmd_handler(ANALOG_INPUT_REQUEST, self._handle_sysex_analog)
        self.add_cmd_handler(DIGITAL_INPUT_REQUEST, self._handle_sysex_digital)
        self.add_cmd_handler(BROADCAST_REPORT, self._handle_sysex_broadcast)

    def add_cmd_handler(self, cmd, func):
        """Adds a command handler for a command."""
        len_args = len(inspect.getargspec(func)[0])
        def add_meta(f):
            def decorator(*args, **kwargs):
                f(*args, **kwargs)
            decorator.bytes_needed = len_args - 1 # exclude self
            decorator.__name__ = f.__name__
            return decorator
        func = add_meta(func)
        self._command_handlers[cmd] = func

    def get_pin(self, pin_def):
        """
        Returns the activated pin given by the pin definition.
        May raise an ``InvalidPinDefError`` or a ``PinAlreadyTakenError``.

        :arg pin_def: Pin definition as described below,
            but without the arduino name. So for example ``a:1:i``.

        'a' analog pin     Pin number   'i' for input
        'd' digital pin    Pin number   'o' for output
                                        'p' for pwm (Pulse-width modulation)

        All seperated by ``:``.
        """
        if type(pin_def) == list:
            bits = pin_def
        else:
            bits = pin_def.split(':')
        a_d = bits[0] == 'a' and 'analog' or 'digital'
        part = getattr(self, a_d)
        pin_nr = int(bits[1])
        if pin_nr >= len(part):
            raise InvalidPinDefError('Invalid pin definition: %s at position 3 on %s' % (pin_def, self.name))
        if getattr(part[pin_nr], 'mode', None)  == UNAVAILABLE:
            raise InvalidPinDefError('Invalid pin definition: UNAVAILABLE pin %s at position on %s' % (pin_def, self.name))
        if self.taken[a_d][pin_nr]:
            raise PinAlreadyTakenError('%s pin %s is already taken on %s' % (a_d, bits[1], self.name))
        # ok, should be available
        pin = part[pin_nr]
        self.taken[a_d][pin_nr] = True
        if pin.type is DIGITAL:
            if bits[2] == 'p':
                pin.mode = PWM
            elif bits[2] == 's':
                pin.mode = SERVO
            elif bits[2] is not 'o':
                pin.mode = INPUT
        else:
            pin.enable_reporting()
        return pin

    def pass_time(self, t):
        """Non-blocking time-out for ``t`` seconds."""
        cont = time.time() + t
        while time.time() < cont:
            time.sleep(0)

    def send_sysex(self, sysex_cmd, data=[]):
        """
        Sends a SysEx msg.

        :arg sysex_cmd: A sysex command byte
        :arg data: A list of 7-bit bytes of arbitrary data (bytes may be
            already converted to chr's)
        """
        self.sp.write(chr(START_SYSEX))
        self.sp.write(chr(sysex_cmd))
        for byte in data:
            try:
                byte = chr(byte)
            except TypeError:
                pass # byte is already a chr
            except ValueError:
                raise ValueError('Sysex data can be 7-bit bytes only. '
                    'Consider using utils.to_two_bytes for bigger bytes.')
            self.sp.write(byte)
        self.sp.write(chr(END_SYSEX))

    def bytes_available(self):
        return self.sp.inWaiting()

    def iterate(self):
        """
        Reads and handles data from the microcontroller over the serial port.
        This method should be called in a main loop or in an :class:`Iterator`
        instance to keep this boards pin values up to date.
        """
        byte = self.sp.read()

        if not byte:
            return

        data = ord(byte)
        received_data = []
        handler = None
        if data < START_SYSEX:
            # These commands can have 'channel data' like a pin nummber appended.
            try:
                handler = self._command_handlers[data & 0xF0]
            except KeyError:
                return
            received_data.append(data & 0x0F)
            while len(received_data) < handler.bytes_needed:
                received_data.append(ord(self.sp.read()))
        elif data == START_SYSEX:
            data = ord(self.sp.read())
            handler = self._command_handlers.get(data)
            if not handler:
                return
            data = ord(self.sp.read())
            while data != END_SYSEX:
                received_data.append(data)
                data = ord(self.sp.read())
        else:
            try:
                handler = self._command_handlers[data]
            except KeyError:
                return
            while len(received_data) < handler.bytes_needed:
                received_data.append(ord(self.sp.read()))
        # Handle the data
        try:
            handler(*received_data)
        except ValueError:
            pass

    def get_firmata_version(self):
        """
        Returns a version tuple (major, minor) for the firmata firmware on the
        board.
        """
        return self.firmata_version

    def servo_config(self, pin, min_pulse=544, max_pulse=2400, angle=0):
        """
        Configure a pin as servo with min_pulse, max_pulse and first angle.
        ``min_pulse`` and ``max_pulse`` default to the arduino defaults.
        """
        if pin > len(self.digital) or self.digital[pin].mode == UNAVAILABLE:
            raise IOError("Pin %s is not a valid servo pin")
        data = itertools.chain([pin], to_two_bytes(min_pulse),
                                        to_two_bytes(max_pulse))
        self.send_sysex(SERVO_CONFIG, data)

        # set pin._mode to SERVO so that it sends analog messages
        # don't set pin.mode as that calls this method
        self.digital[pin]._mode = SERVO
        self.digital[pin].write(angle)

    def exit(self):
        """Call this to exit cleanly."""
        # First detach all servo's, otherwise it somehow doesn't want to close...
        if hasattr(self, 'digital'):
            for pin in self.digital:
                if pin.mode == SERVO:
                    pin.mode = OUTPUT
        if hasattr(self, 'sp'):
            self.sp.close()
        self.running = 0

    # Command handlers
    def _handle_analog_message(self, pin_nr, lsb, msb):
        value = round(float((msb << 7) + lsb) / 1023, 4)
        # Only set the value if we are actually reporting
        try:
            if self.analog[pin_nr].reporting:
                self.analog[pin_nr].value = value
        except IndexError:
            raise ValueError

    def _handle_digital_message(self, port_nr, lsb, msb):
        """
        Digital messages always go by the whole port. This means we have a
        bitmask which we update the port.
        """
        mask = (msb << 7) + lsb
        try:
            self.digital_ports[port_nr]._update(mask)
        except IndexError:
            raise ValueError

    def _handle_report_version(self, major, minor):
        self.firmata_version = (major, minor)

    def _handle_report_firmware(self, *data):
        major = data[0]
        minor = data[1]
        self.firmware_version = (major, minor)
        self.firmware = two_byte_iter_to_str(data[2:])

    def _handle_sysex_ping(self, *data):
        major = data[0]
        minor = data[1]
        robot = data[2]
        self.nearest_obstacle[robot] = minor + major * 128

    def _handle_sysex_analog(self, *data):
        major = data[0]
        minor = data[1]
        robot = data[2]
        self.analog_value[robot] = minor + major * 128

    def _handle_sysex_digital(self, *data):
        major = data[0]
        robot = data[1]
        self.digital_value[robot] = major

    def _handle_sysex_broadcast(self, *data):
        robot = data[0]
        self.live_robots[robot] = 1


class _WrapTCPSocket(object):
    def __init__(self, socket):
        self.skt = socket

    def read(self):
        if self.inWaiting():
            return self.skt.recv(1)
        return ''

    def write(self, data):
        self.skt.sendall(data)

    def inWaiting(self):
        inputready, _, _ = select.select([self.skt.fileno()], [], [], 0)
        return 1 if inputready else 0

    def close(self):
        self.skt.close()


class TCPBoard(Board):
    def __init__(self, robot_ip, port, layout, name=None):
        # Se crea el socket
        self.skt = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        # Se crea la conexión
        self.skt.connect((robot_ip, port))
        # Wrapper que simula una instancia de Serial
        self.sp = _WrapTCPSocket(self.skt)
        self.ip = robot_ip
        self.port = port
        if name is None:
            self.name = "{}:{}".format(robot_ip, port)
        else:
            self.name = name
        self._generic_initialization(layout, name)


class Port(object):
    """An 8-bit port on the board."""
    def __init__(self, board, port_number, num_pins=8):
        self.board = board
        self.port_number = port_number
        self.reporting = False

        self.pins = []
        for i in range(num_pins):
            pin_nr = i + self.port_number * 8
            self.pins.append(Pin(self.board, pin_nr, type=DIGITAL, port=self))

    def __str__(self):
        return "Digital Port %i on %s" % (self.port_number, self.board)

    def enable_reporting(self):
        """Enable reporting of values for the whole port."""
        self.reporting = True
        msg = chr(REPORT_DIGITAL + self.port_number)
        msg += chr(1)
        self.board.sp.write(msg)
        for pin in self.pins:
            if pin.mode == INPUT:
                pin.reporting = True  # TODO Shouldn't this happen at the pin?

    def disable_reporting(self):
        """Disable the reporting of the port."""
        self.reporting = False
        msg = chr(REPORT_DIGITAL + self.port_number)
        msg += chr(0)
        self.board.sp.write(msg)

    def write(self):
        """Set the output pins of the port to the correct state."""
        mask = 0
        for pin in self.pins:
            if pin.mode == OUTPUT:
                if pin.value == 1:
                    pin_nr = pin.pin_number - self.port_number * 8
                    mask |= 1 << pin_nr
        msg = chr(DIGITAL_MESSAGE + self.port_number)
        msg += chr(mask % 128)
        msg += chr(mask >> 7)
        self.board.sp.write(msg)

    def _update(self, mask):
        """Update the values for the pins marked as input with the mask."""
        if self.reporting:
            for pin in self.pins:
                if pin.mode is INPUT:
                    pin_nr = pin.pin_number - self.port_number * 8
                    pin.value = (mask & (1 << pin_nr)) > 0

class Pin(object):
    """A Pin representation"""
    def __init__(self, board, pin_number, type=ANALOG, port=None):
        self.board = board
        self.pin_number = pin_number
        self.type = type
        self.port = port
        self.PWM_CAPABLE = False
        self._mode = (type == DIGITAL and OUTPUT or INPUT)
        self.reporting = False
        self.value = None

    def __str__(self):
        type = {ANALOG : 'Analog', DIGITAL : 'Digital'}[self.type]
        return "%s pin %d" % (type, self.pin_number)

    def _set_mode(self, mode):
        if mode is UNAVAILABLE:
            self._mode = UNAVAILABLE
            return
        if self._mode is UNAVAILABLE:
            raise IOError("%s can not be used through Firmata." % self)
        if mode is PWM and not self.PWM_CAPABLE:
            raise IOError("%s does not have PWM capabilities." % self)
        if mode == SERVO:
            if self.type != DIGITAL:
                raise IOError("Only digital pins can drive servos! %s is not"
                    "digital." % self)
            self._mode = SERVO
            self.board.servo_config(self.pin_number)
            return

        # Set mode with SET_PIN_MODE message
        self._mode = mode
        command = chr(SET_PIN_MODE)
        command += chr(self.pin_number)
        command += chr(mode)
        self.board.sp.write(command)
        if mode == INPUT:
            self.enable_reporting()

    def _get_mode(self):
        return self._mode

    mode = property(_get_mode, _set_mode)
    """
    Mode of operation for the pin. Can be one of the pin modes: INPUT, OUTPUT,
    ANALOG, PWM. or SERVO (or UNAVAILABLE).
    """

    def enable_reporting(self):
        """Set an input pin to report values."""
        if self.mode is not INPUT:
            raise IOError, "%s is not an input and can therefore not report" % self
        if self.type == ANALOG:
            self.reporting = True
            msg = chr(REPORT_ANALOG + self.pin_number)
            msg += chr(1)
            self.board.sp.write(msg)
        else:
            self.port.enable_reporting() # TODO This is not going to work for non-optimized boards like Mega

    def disable_reporting(self):
        """Disable the reporting of an input pin."""
        if self.type == ANALOG:
            self.reporting = False
            msg = chr(REPORT_ANALOG + self.pin_number)
            msg += chr(0)
            self.board.sp.write(msg)
        else:
            self.port.disable_reporting() # TODO This is not going to work for non-optimized boards like Mega

    def read(self):
        """
        Returns the output value of the pin. This value is updated by the
        boards :meth:`Board.iterate` method. Value is always in the range from
        0.0 to 1.0.
        """
        if self.mode == UNAVAILABLE:
            raise IOError, "Cannot read pin %s"% self.__str__()
        return self.value

    def write(self, value):
        """
        Output a voltage from the pin

        :arg value: Uses value as a boolean if the pin is in output mode, or
            expects a float from 0 to 1 if the pin is in PWM mode. If the pin
            is in SERVO the value should be in degrees.

        """
        if self.mode is UNAVAILABLE:
            raise IOError, "%s can not be used through Firmata." % self
        if self.mode is INPUT:
            raise IOError, "%s is set up as an INPUT and can therefore not be written to" % self
        if value is not self.value:
            self.value = value
            if self.mode is OUTPUT:
                if self.port:
                    self.port.write()
                else:
                    msg = chr(DIGITAL_MESSAGE)
                    msg += chr(self.pin_number)
                    msg += chr(value)
                    self.board.sp.write(msg)
            elif self.mode is PWM:
                value = int(round(value * 255))
                msg = chr(ANALOG_MESSAGE + self.pin_number)
                msg += chr(value % 128)
                msg += chr(value >> 7)
                self.board.sp.write(msg)
            elif self.mode is SERVO:
                value = int(value)
                msg = chr(ANALOG_MESSAGE + self.pin_number)
                msg += chr(value % 128)
                msg += chr(value >> 7)
                self.board.sp.write(msg)
